VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "clsIConnectionPoint"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
' Autor: Frank Schler
' Datum: 09.08.2018
' Status: Komplett

Option Explicit

' ----==== Const ====----
Private Const S_OK As Long = &H0&
Private Const CLSID_ConnectionPoint As String = ""
Private Const IID_IConnectionPoint As String = "{B196B284-BAB4-101A-B69C-00AA00341D07}"

Private Enum E_vtbOffsets
    ' /*** IUnknown methods ***/
    vtb_QueryInterface
    vtb_AddRef
    vtb_Release
    ' /*** IConnectionPoint methods ***/
    vtb_GetConnectionInterface
    vtb_GetConnectionPointContainer
    vtb_Advise
    vtb_Unadvise
    vtb_EnumConnections
End Enum

' ----==== Ole32 API-Deklarationen ====----
Private Declare Function IIDFromString Lib "ole32" ( _
                         ByVal lpsz As Long, _
                         ByVal lpiid As Long) As Long

' ----==== Variablen ====----
Private m_Interface As InterfaceData

' ----==== IUnknown Func ====----
'
'    /// <summary>
'    /// Retrieves pointers to the supported interfaces on an object.
'    /// </summary>
'   HRESULT QueryInterface(
'       REFIID riid,
'       void   **ppvObject
'       );
Public Function QueryInterface(ByVal riid As String) As Long

    Dim lRet As Long
    Dim triid(3) As Long
    Dim psriid As Long
    Dim ptriid As Long
    Dim ppvObject As Long
    Dim pppvObject As Long

    lRet = 0
    
    ' Pointer
    psriid = StrPtr(riid)
    ptriid = VarPtr(triid(0))

    ' IID -> triid
    If IIDFromString(psriid, ptriid) = S_OK Then
    
        ' Pointer
        pppvObject = VarPtr(ppvObject)

        ' QueryInterface des Interfaces aufrufen
        If x_OleInvoke(vtb_QueryInterface, ptriid, pppvObject) = S_OK Then

            lRet = ppvObject
        
        End If

    End If

    QueryInterface = lRet

End Function

' ----==== IConnectionPoint Func ====----
'
Friend Function GetConnectionInterface() As GUID

    Dim tRet As GUID
    Dim pIID As GUID
    Dim ppIID As Long

    ppIID = VarPtr(pIID)

    If x_OleInvoke(vtb_GetConnectionInterface, ppIID) = S_OK Then

        tRet = pIID

    End If

    GetConnectionInterface = tRet

End Function

Public Function GetConnectionPointContainer() As Long

    Dim pRet As Long
    Dim pCPC As Long
    Dim ppCPC As Long

    ppCPC = VarPtr(pCPC)

    If x_OleInvoke(vtb_GetConnectionPointContainer, ppCPC) = S_OK Then

        pRet = pCPC

    End If

    GetConnectionPointContainer = pRet

End Function

Public Function Advise(ByVal pUnkSink As Long) As Long

    Dim pRet As Long
    Dim pdwCookie As Long
    Dim ppdwCookie As Long
    
    ppdwCookie = VarPtr(pdwCookie)
    
    If x_OleInvoke(vtb_Advise, pUnkSink, ppdwCookie) = S_OK Then

        pRet = pdwCookie

    End If

    Advise = pRet
    
End Function

Public Function Unadvise(ByVal dwCookie As Long) As Boolean

    Dim bRet As Boolean
    
    bRet = False
    
    If x_OleInvoke(vtb_Unadvise, dwCookie) = S_OK Then

        bRet = True

    End If

    Unadvise = bRet
    
End Function

Public Function EnumConnections() As Long

    Dim pRet As Long
    Dim pEnum As Long
    Dim ppEnum As Long
    
    ppEnum = VarPtr(pEnum)
    
    If x_OleInvoke(vtb_EnumConnections, ppEnum) = S_OK Then

        pRet = pEnum

    End If

    EnumConnections = pRet
    
End Function

' ----==== Interface Helper Func ====----
'
' Initialisiert das Interface ber einen Interface-Pointer
Public Sub Init(ByVal pInterface As Long)

    If pInterface <> 0& Then

        m_Interface.pObject = ObjPtr(Me)
        m_Interface.boolRaiseErrors = False
        m_Interface.pInterface = pInterface

    End If

End Sub

' Ruft Release des Interfaces auf
Private Sub Class_Terminate()

    Call ReleaseInterface(m_Interface)

End Sub

' Fhrt die Funktion des Interfaces aus
Private Function x_OleInvoke(ByVal vtb As E_vtbOffsets, ParamArray var()) As Variant
    
    Dim varRet As Variant
    
    ' ist ein Pointer auf ein Interface vorhanden
    If m_Interface.pInterface <> 0& Then

        ' Funktion ausfhren
        varRet = OleInvoke(m_Interface, vtb, var)
        
        ' nur falls ein Feler auftritt
        If varRet <> S_OK Then
        
            Debug.Print TypeName(Me) & ": E_vtbOffsets = " & CStr(vtb)
        
        End If
        
        x_OleInvoke = varRet

    End If

End Function

' wird aus der modInterface bei einem Fehler aufgerufen
Public Sub x_RaiseError(ByVal lngErrCode As Long, ByVal strErrText As String)

    Call Err.Raise(lngErrCode, App.EXEName & "\" & TypeName(Me), strErrText)

End Sub

' Fehler auslsen? Standard = False
Public Property Let RaiseErrors(ByVal Raise As Boolean)

    m_Interface.boolRaiseErrors = Raise

End Property

' liefert den Pointer auf das Interface zurck
Public Property Get IfcPtr() As Long

    IfcPtr = m_Interface.pInterface

End Property

