Attribute VB_Name = "modINetworkEvents"
' Autor: Frank Schler
' Datum: 09.12.2019
' Status: Komplett
            
Option Explicit

' ----==== Const ====----
Private Const S_OK As Long = &H0
Private Const E_NOINTERFACE As Long = &H80004002
Private Const PAGE_EXECUTE_READWRITE As Long = &H40
Private Const IID_IUnknown As String = "{00000000-0000-0000-C000-000000000046}"
Private Const IID_INetworkEvents As String = "{DCB00004-570F-4A9B-8D69-199FDBA5723B}"

' ----==== Types ====----
Private Type tIUnknown
    pVTable As Long
End Type

Private Type tIUnknownVTable
    VTable(0 To 2) As Long
End Type

Private Type tINetworkEvents
    pVTable As Long
End Type

Private Type tINetworkEventsVTable
    VTable(0 To 6) As Long
End Type

' ----==== Kernel32 API-Deklarationen ====----
Private Declare Sub CopyMemory Lib "kernel32" _
                    Alias "RtlMoveMemory" ( _
                    ByRef Destination As Any, _
                    ByRef Source As Any, _
                    ByVal length As Long)

' ----==== Variablen ====----
Private m_IPtr As Long
Private m_RefCount As Long
Private m_Interface As Object
Private m_IUnknown As tIUnknown
Private m_IUnknownVTable As tIUnknownVTable
Private m_INetworkEvents As tINetworkEvents
Private m_INetworkEventsVTable As tINetworkEventsVTable

Public Function INetworkEvents(ByVal cINetworkEvents As Long) As Long

    ' Objekt vom Pointer
    Call CopyMemory(m_Interface, cINetworkEvents, 4)
    
    ' ----==== Interface IUnknown  ====----
    With m_IUnknownVTable
        
        ' VTable aufbauen
        .VTable(0) = ProcPtr(AddressOf QueryInterface)
        .VTable(1) = ProcPtr(AddressOf AddRef)
        .VTable(2) = ProcPtr(AddressOf Release)

    End With

    With m_IUnknown

        ' Pointer auf die VTable
        .pVTable = VarPtr(m_IUnknownVTable)

    End With
    
    ' ----==== Interface INetworkEvents  ====----
    With m_INetworkEventsVTable
        
        ' VTable aufbauen
        .VTable(0) = ProcPtr(AddressOf QueryInterface)
        .VTable(1) = ProcPtr(AddressOf AddRef)
        .VTable(2) = ProcPtr(AddressOf Release)
        .VTable(3) = ProcPtr(AddressOf NetworkAdded)
        .VTable(4) = ProcPtr(AddressOf NetworkDeleted)
        .VTable(5) = ProcPtr(AddressOf NetworkConnectivityChanged)
        .VTable(6) = ProcPtr(AddressOf NetworkPropertyChanged)

    End With

    With m_INetworkEvents

        ' Pointer auf die VTable
        .pVTable = VarPtr(m_INetworkEventsVTable)

    End With
    
    
    ' Pointer auf das Interface
    m_IPtr = VarPtr(m_IUnknown)

    INetworkEvents = m_IPtr

End Function

Private Function ProcPtr(ByVal ptr As Long) As Long

    ProcPtr = ptr

End Function

' ----==== Interface IUnknown Func ====----
Private Function QueryInterface(ByVal this As Long, ByRef riid As GUID, ByRef _
    pvObj As Long) As Long

    Dim lRet As Long
    
    Select Case UCase$(Guid2String(riid))

    ' wenn nach dem Interface IUnknown gefragt wird
    Case UCase$(IID_IUnknown)
    
        ' dann muss ein AddRef aufgerufen werden.
        Call AddRef(this)
        
        ' Pointer auf das Interface
        pvObj = VarPtr(m_IUnknown)
        
        ' OK zurck geben, Interface ist vorhanden
        lRet = S_OK
    
    ' wenn nach dem Interface INetworkEvents gefragt wird
    Case UCase$(IID_INetworkEvents)

        ' dann muss ein AddRef aufgerufen werden.
        Call AddRef(this)
        
        ' Pointer auf das Interface
        pvObj = VarPtr(m_INetworkEvents)
        
        ' OK zurck geben, Interface ist vorhanden
        lRet = S_OK

    Case Else
        
        ' alles andere ignorieren
        
        pvObj = 0&
        
        lRet = E_NOINTERFACE
    
    End Select

    QueryInterface = lRet

End Function

Private Function AddRef(ByVal this As Long) As Long

    m_RefCount = m_RefCount + 1

    AddRef = m_RefCount

End Function

Private Function Release(ByVal this As Long) As Long
    
    m_RefCount = m_RefCount - 1
    
    If m_RefCount = 0 Then
    
        ' alles Aufrumen und Freigeben
        m_IUnknown.pVTable = 0&
        Erase m_IUnknownVTable.VTable
                
        m_INetworkEvents.pVTable = 0&
        Erase m_INetworkEventsVTable.VTable
        
        m_IPtr = 0&
        
        ' Objekt lschen
        Call CopyMemory(m_Interface, 0&, 4)
        
    End If
    
    Release = m_RefCount

End Function

' ----==== Interface INetworkEvents ====----
'
' anstelle von "ByVal GUID" verwenden wir hier 4 Longs
'
Private Function NetworkAdded(ByVal this As Long, ByVal networkID1 As Long, _
    ByVal networkID2 As Long, ByVal networkID3 As Long, ByVal networkID4 As Long) As Long
    
    Dim tGUIDL As GUIDL
    
    With tGUIDL
        .Data1 = networkID1
        .Data2 = networkID2
        .Data3 = networkID3
        .Data4 = networkID4
    End With
    
    
    ' Funktion in der Klasse aufrufen
    Call m_Interface.NetworkAdded(GuidL2String(tGUIDL))

End Function

' anstelle von "ByVal GUID" verwenden wir hier 4 Longs
'
Private Function NetworkDeleted(ByVal this As Long, ByVal networkID1 As Long, _
    ByVal networkID2 As Long, ByVal networkID3 As Long, ByVal networkID4 As Long) As Long
    
    Dim tGUIDL As GUIDL
    
    With tGUIDL
        .Data1 = networkID1
        .Data2 = networkID2
        .Data3 = networkID3
        .Data4 = networkID4
    End With
    
    ' Funktion in der Klasse aufrufen
    Call m_Interface.NetworkDeleted(GuidL2String(tGUIDL))

End Function

' anstelle von "ByVal GUID" verwenden wir hier 4 Longs
'
Private Function NetworkConnectivityChanged(ByVal this As Long, ByVal networkID1 As Long, _
    ByVal networkID2 As Long, ByVal networkID3 As Long, ByVal networkID4 As Long, _
    ByVal newConnectivity As Long) As Long
    
    Dim tGUIDL As GUIDL
    
    With tGUIDL
        .Data1 = networkID1
        .Data2 = networkID2
        .Data3 = networkID3
        .Data4 = networkID4
    End With
    
    ' Funktion in der Klasse aufrufen
    Call m_Interface.NetworkConnectivityChanged(GuidL2String(tGUIDL), newConnectivity)

End Function

' anstelle von "ByVal GUID" verwenden wir hier 4 Longs
'
Private Function NetworkPropertyChanged(ByVal this As Long, ByVal networkID1 As Long, _
    ByVal networkID2 As Long, ByVal networkID3 As Long, ByVal networkID4 As Long, _
    ByVal flags As Long) As Long
    
    Dim tGUIDL As GUIDL
    
    With tGUIDL
        .Data1 = networkID1
        .Data2 = networkID2
        .Data3 = networkID3
        .Data4 = networkID4
    End With
    
    ' Funktion in der Klasse aufrufen
    Call m_Interface.NetworkPropertyChanged(GuidL2String(tGUIDL), flags)

End Function

